#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import urllib

from module.utils import html_unescape
from module.plugins.Hoster import Hoster

class YoutubeCom(Hoster):
    __name__ = "YoutubeCom"
    __type__ = "hoster"
    __pattern__ = r"http://(www\.)?(de\.)?\youtube\.com/watch\?v=.*"
    __version__ = "0.2"
    __config__ = [("quality", "sd;hd;fullhd", "Quality Setting", "hd"),
            ("fmt", "int", "FMT Number 0-45", 0),
            (".mp4", "bool", "Allow .mp4", True),
            (".flv", "bool", "Allow .flv", True),
            (".webm", "bool", "Allow .webm", False),
            (".3gp", "bool", "Allow .3gp", False)]
    __description__ = """Youtube.com Video Download Hoster"""
    __author_name__ = ("spoob")
    __author_mail__ = ("spoob@pyload.org")

    # name, width, height, quality ranking
    formats = {17: (".3gp", 176, 144, 0),
               5: (".flv", 400, 240, 1),
               18: (".mp4", 480, 360, 2),
               43: (".webm", 640, 360, 3),
               34: (".flv", 640, 360, 4),
               35: (".flv", 854, 480, 5),
               45: (".webm", 1280, 720, 6),
               22: (".mp4", 1280, 720, 7),
               37: (".mp4", 1920, 1080, 8),
               38: (".mp4", 4096, 3072, 9),
               }


    def process(self, pyfile):
        html = self.load(pyfile.url, decode=True)

        if "watch-player-unavailable" in html:
            self.offline()

        videoId = pyfile.url.split("v=")[1].split("&")[0]
        videoHash = re.search(r'&amp;t=(.+?)&', html).group(1)

        file_name_pattern = '<meta name="title" content="(.+?)">'

        quality = self.getConf("quality")
        desired_fmt = 18

        if quality == "sd":
            desired_fmt = 18
        elif quality == "hd":
            desired_fmt = 22
        elif quality == "fullhd":
            desired_fmt = 37

        if self.getConfig("fmt"):
            desired_fmt = self.getConf("fmt")

        fmt_pattern = 'fmt_url_map=(.+?)&'
        fmt_url_map = re.search(fmt_pattern, html).group(1)
        links = urllib.unquote(fmt_url_map).split(",")

        fmt_dict = {}
        for link in links:
            fmt = link.split("|")[0]
            try:
                fmt = int(fmt)
            except Exception:
                continue

            fmt_dict[fmt] = link.split("|")[1]

        self.logDebug("Found links: %s" % fmt_dict)

        allowed = lambda x: self.getConfig(self.formats[x][0])
        sel = lambda x: self.formats[x][3] #select quality index
        comp = lambda x, y: abs(sel(x) - sel(y))

        #return fmt nearest to quali index
        fmt = reduce(lambda x, y: x if comp(x, desired_fmt) <= comp(y, desired_fmt) and
                                       sel(x) > sel(y) and
                                       allowed(x) else y, fmt_dict.keys())

        self.logDebug("Choose fmt: %s" % fmt)

        file_suffix = ".flv"
        if fmt in self.formats:
            file_suffix = self.formats[fmt][0]
        name = re.search(file_name_pattern, html).group(1).replace("/", "") + file_suffix
        pyfile.name = html_unescape(name)

        self.download(fmt_dict[fmt])
